package com.ruoyi.web.aop;

import com.ruoyi.common.utils.DateUtil;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.crops.domain.InterfaceStatistics;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import java.lang.reflect.Array;

import java.time.Duration;
import java.util.*;

/**
 * API访问历史统计
 * @author yangjunqiang
 */

@Aspect
@Component
public class ApiVisitHistory {

    private Logger log = LoggerFactory.getLogger(ApiVisitHistory.class);

    private ThreadLocal<Long> startTime = new ThreadLocal<>();

    private Set userIds = new HashSet();

    @Autowired
    private RedisTemplate redisTemplate;
    /**
     * 定义切面
     * - 此处代表com.ruoyi.web.controller.crops包下的所有接口都会被统计
     */
    @Pointcut("execution(* com.ruoyi.web.controller.crops..*.*(..))")
    public void pointCut(){

    }

    @Before("pointCut()")
    public void doBefore(JoinPoint joinPoint) {
        startTime.set(System.currentTimeMillis());
        //计数
        AtomicCounter.getInstance().increase();
    }

    @AfterReturning(returning = "returnVal", pointcut = "pointCut()")
    public void doAfterReturning(JoinPoint joinPoint, Object returnVal) {
        //判断访问接口的用户数
        try {
            userIds.add(SecurityUtils.getUserId());
        }catch (Exception e){}
        //从redis中获取数据
        InterfaceStatistics iS = (InterfaceStatistics) redisTemplate.opsForValue().get("interfaceStatistics");
        //当前时间和今天24点相差毫秒数
        long millis = DateUtil.dateDiff(new Date(), DateUtil.getDayEnd());
        //如果redis中有数据
        if (iS!=null){
            Integer visits = iS.getVisits();
            Integer dataEntries = iS.getDataEntries();
            dataEntries +=getLength(returnVal);
            Integer accessingUsers = iS.getAccessingUsers();
//        log.info("耗费时间:[{}] ms,访问次数：{},交换数据总量:{}", System.currentTimeMillis() - startTime.get(),AtomicCounter.getInstance().getValue(),dataEntries);
            InterfaceStatistics interfaceStatistics = new InterfaceStatistics();
            //访问次数进行自增
            interfaceStatistics.setVisits(++visits);
            //数据总量
            interfaceStatistics.setDataEntries(dataEntries);
            //用户访问数，只取最大
            interfaceStatistics.setAccessingUsers((userIds.size()>accessingUsers)?userIds.size():accessingUsers);
            redisTemplate.opsForValue().set("interfaceStatistics",interfaceStatistics, Duration.ofMillis(millis));
        }else {
            InterfaceStatistics interfaceStatistics = new InterfaceStatistics();
            interfaceStatistics.setVisits(1);
            interfaceStatistics.setDataEntries(1);
            interfaceStatistics.setAccessingUsers(1);
            redisTemplate.opsForValue().set("interfaceStatistics",interfaceStatistics, Duration.ofMillis(millis));
        }
    }

    /**
     * 当接口报错时执行此方法
     */
    @AfterThrowing(pointcut = "pointCut()")
    public void doAfterThrowing(JoinPoint joinPoint) {
        log.info("接口访问失败");
    }


    //判断Object 的类型(map,set,list,array),并返回长度
    private static int getLength(Object obj) {
        if (obj == null) {
            return 0;
        }
        if (obj.getClass().isArray()){
            int length = Array.getLength(obj);
            return length;
        }if (obj instanceof List) {
            return ((List) obj).size();
        }if (obj instanceof Set) {
            return ((Set) obj).size();
        }if (obj instanceof Map) {
            Object data = ((Map) obj).get("data");
            getLength(data);
        }
        return 1;
    }

}
