package com.ruoyi.web.aop;

import java.util.concurrent.atomic.AtomicInteger;

/**
 * 计数器，统计当前执行的任务数
 * @author yangjunqiang
 *
 */
public class AtomicCounter {

	/**
	 * 创建了一个私有静态的AtomicCounter对象，作为单例模式的实例。
	 */
	private static final AtomicCounter atomicCounter = new AtomicCounter();

	/**
	 * 私有的构造函数，用于限制类的实例化。其他类无法直接实例化该类，只能通过getInstance()方法获取单例对象
	 */
	private AtomicCounter() {

	}

	/**
	 * 公共静态方法getInstance()，用于获取AtomicCounter类的单例对象。
	 * @return 返回之前创建的静态atomicCounter对象
	 */
	public static AtomicCounter getInstance() {
		return atomicCounter;
	}

	/**
	 * 创建了一个私有静态的AtomicInteger对象counter，用于存储计数器的值。
	 */
	private static AtomicInteger counter = new AtomicInteger();

	/**
	 * 公共方法getValue()，用于获取计数器的当前值。
	 * @return 通过调用counter.get()来获取counter的当前值，并返回。
	 */
	public int getValue() {
		return counter.get();
	}

	/**
	 * 公共方法increase()，用于递增计数器的值。
	 * @return 通过调用counter.incrementAndGet()来递增counter的值，并返回递增后的新值。
	 */
	public int increase() {
		return counter.incrementAndGet();
	}

	/**
	 * 公共方法decrease()，用于递减计数器的值。
	 * 通过调用counter.decrementAndGet()来递减counter的值，并返回递减后的新值。
	 * @return
	 */
	public int decrease() {
		return counter.decrementAndGet();
	}

}
