package com.ruoyi.web.controller.crops;

import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.crops.domain.CropsUser;
import com.ruoyi.crops.service.ICropsUserService;

import org.springframework.beans.factory.annotation.Autowired;

import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.Errors;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpSession;
import javax.validation.Valid;
import java.util.List;

/**
 * 用户controller
 */

@RestController
@RequestMapping("/crops/user")
public class CropsUserController extends BaseController {
    @Autowired
    private ICropsUserService iCropsUserService;

    /**
     * 根据id查询用户
     * @param phone
     * @return
     */
    @GetMapping("/{phone}")
    public CropsUser selectByPhone(@PathVariable String phone) {
        return iCropsUserService.selectByPhone(phone);
    }

    /**
     * 获取所有用户列表
     *
     * @return
     */
    @PreAuthorize("@ss.hasPermi('crops:user:list')")
    @GetMapping()
    public List<CropsUser> list() {
        return iCropsUserService.list();
    }

    /**
     * 删除用户
     *
     * @param id
     * @return
     */
    @PreAuthorize("@ss.hasPermi('crops:user:delete')")
    @DeleteMapping("/{id}")
    public AjaxResult delete(@PathVariable Integer id) {
        return toAjax(iCropsUserService.delete(id));
    }

    /**
     * 注册用户
     *
     * @param cropsUser
     * @return
     */
    @PostMapping("/register")
    public AjaxResult register(@Valid CropsUser cropsUser, Errors errors, String psw2) {
        if (errors.hasErrors()) {
            return AjaxResult.error(errors.getAllErrors().get(0).getDefaultMessage());
        } else if (!cropsUser.getPassword().equals(psw2)) {
            return AjaxResult.error("两次密码不同");
        } else {
            try {
                int ret = iCropsUserService.regist(cropsUser);
                if (ret > 0) {
                    return AjaxResult.success(cropsUser);
                } else {
                    return AjaxResult.error("注册失败");
                }
            } catch (Exception ex) {
                return AjaxResult.error(ex.getMessage());
            }
        }
    }

    @PostMapping("/login")
    public AjaxResult login(@Valid CropsUser cropsUser, Errors errors, HttpSession session) {
        if (errors.hasErrors()) {
            return AjaxResult.error(errors.getAllErrors().get(0).getDefaultMessage());
        } else {
            CropsUser user = iCropsUserService.login(cropsUser);
            if (user != null) {
                session.setAttribute("cropsUser", user);
                return AjaxResult.success("登录成功");
            } else {
                return AjaxResult.error("用户不存在/密码错误");
            }
        }
    }

    /**
     * 修改用户
     *
     * @param id
     * @return
     */
    @PutMapping("/{id}")
    public AjaxResult update(@PathVariable Integer id) {
        return toAjax(iCropsUserService.update(id));
    }
}
