package com.ruoyi.web.controller.crops;

import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.google.gson.Gson;

import com.google.gson.GsonBuilder;
import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.utils.DateUtil;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ShapeUtil;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.crops.domain.*;
import com.ruoyi.common.utils.MyPoint;
import com.ruoyi.crops.domain.gsonBean.GsonFarmMachineryBean;
import com.ruoyi.crops.domain.gsonBean.GsonMachineryPositionBean;
import com.ruoyi.crops.domain.vo.FarmMachineryVo;
import com.ruoyi.crops.domain.vo.MachineryJobDataVo;
import com.ruoyi.crops.service.*;

import org.apache.commons.collections.map.HashedMap;
import org.checkerframework.checker.units.qual.A;
import org.locationtech.jts.geom.Geometry;
import org.locationtech.jts.geom.GeometryFactory;
import org.locationtech.jts.geom.MultiPolygon;
import org.locationtech.jts.geom.Polygon;
import org.locationtech.jts.io.WKTReader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.bind.annotation.*;

import java.io.File;
import java.io.IOException;
import java.text.ParseException;

import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.util.*;

@RestController
@RequestMapping("/farm/machinery")
public class FarmMachineryController extends BaseController {
    private static final Logger logger = LoggerFactory.getLogger(FarmMachineryController.class);
    @Autowired
    private IFarmMachineryService iFarmMachineryService;
    @Autowired
    private IMapDrawPolygonService iMapDrawPolygonService;
    @Autowired
    private IWarningFarmService iWarningFarmService;
    @Autowired
    private IGeoserverShpService iGeoserverShpService;
    @Autowired
    private IGeoServer iGeoServer;

    @Value("${FarmMachinery.vehicleList}")
    private String vehicleListUrl;
    @Value("${FarmMachinery.vehiclePos}")
    private String vehiclePosUrl;
    @Value("${FarmMachinery.vehicleTrack}")
    private String vehicleTrackUrl;
    @Value("${FarmMachinery.vehicleJobData}")
    private String vehicleJobDataUrl;

    @Value("${FarmMachinery.clienttoken}")
    private String clienttoken;


    /**
     * 农机设备信息查询
     *
     * @return
     */
    @GetMapping("selectMachineryData")
    public TableDataInfo selectMachineryData() {
        startPage();
        List<FarmMachineryVo> list = iFarmMachineryService.selectMachineryData();
        return getDataTable(list);
    }

    /**
     * 农机位置查询
     *
     * @param vehicleno
     * @return
     */
    @GetMapping("/position")
    public JSONObject machineryPosition(@RequestParam(value = "vehicleno", required = false) String vehicleno) {
        Map<String, Object> map = new HashMap<>();//存放参数
        map.put("clienttoken", clienttoken);
        if (vehicleno != null) {
            map.put("vehicleno", vehicleno);
        }
        String result = HttpUtil.createPost(vehiclePosUrl).form(map).execute().body();
        Gson gson = new GsonBuilder().setDateFormat("yyyy-MM-dd HH:mm:ss").create();
        GsonMachineryPositionBean gsonMachineryPositionBean = gson.fromJson(result, GsonMachineryPositionBean.class);
        List<MachineryPosition> position = gsonMachineryPositionBean.getPosition();
        if (position.size()>0){
            iFarmMachineryService.insertPositions(position);
            WithinAndIntersects(position);
        }
        return JSONObject.parseObject(result);
    }
    //判断农机经纬度是否在范围内
    public void WithinAndIntersects(List<MachineryPosition> position) {
        List<Map<String, Object>> maps = null;
        try {
            //读取上传的shp文件
            maps = ShapeUtil.readShp(RuoYiConfig.getUploadPath() + "/shpFile/shapefile.shp");
        } catch (IOException e) {
            e.printStackTrace();
        }
        for (Map<String, Object> map : maps) {
            String wkt = (String) map.get("wkt");
            //更改字符串格式 11,22|22,33|33,22|11,22
            String substring = wkt.substring(16, wkt.length() - 3);
            String replace = substring.replace(", ", "|").replace(" ", ",");
            for (MachineryPosition machineryPosition : position) {
                if (!iFarmMachineryService.withinAndIntersects(replace, machineryPosition.getLng(), machineryPosition.getLat())){
                    WarningFarm warningFarm = new WarningFarm();
                    String vehicleno = machineryPosition.getVehicleno();
                    warningFarm.setVehicleno(vehicleno);
                    FarmMachineryVo farmMachineryVos = iFarmMachineryService.selectMachineryData(vehicleno);
                    warningFarm.setOwnername(farmMachineryVos.getOwnername());
                    warningFarm.setTime(new Date());
                    warningFarm.setWarningInfo("作业范围超出");
                    iWarningFarmService.insertWarning(warningFarm);
                }
            }
        }
    }

    /**
     * 历史作业信息查询
     *
     * @return
     */
    @GetMapping("/selectMachineryJobData")
    public TableDataInfo selectMachineryJobData(@RequestParam(required = false, name = "startTime") Date startTime,
                                                @RequestParam(required = false, name = "endTime") Date endTime,
                                                @RequestParam(required = false, name = "vehicleno") String vehicleno) {
        startPage();
        List<MachineryJobDataVo> list = iFarmMachineryService.selectMachineryJobData(startTime, endTime, vehicleno);
        return getDataTable(list);
    }

    /**
     * 查询全部农机编号
     * @return
     */
    @GetMapping("/selectVehicleno")
    public AjaxResult selectVehicleno(){
        List<String> list =  iFarmMachineryService.selectVehicleno();
        return AjaxResult.success(list);
    }

    /**
     * 今日作业查询
     *
     * @return
     */
    @GetMapping("/selectTodayJobData")
    public AjaxResult selectTodayJobData() {
        MachineryJobDataVo data = iFarmMachineryService.selectTodayJobData();
        Map<String, Double> map = new HashedMap();
        map.put("area", data.getArea());
        map.put("workhours", data.getWorkhours());
        return AjaxResult.success(map);
    }

    /**
     * 农机作业信息导出
     *
     * @param
     */
//    @Log(title = "农机作业信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public AjaxResult export(@RequestParam("startTime") Date startTime,
                             @RequestParam("endTime") Date endTime,
                             @RequestParam("vehicleno") String vehicleno) {
        List<MachineryJobDataVo> list = iFarmMachineryService.selectMachineryJobData(startTime, endTime, vehicleno);
        ExcelUtil<MachineryJobDataVo> util = new ExcelUtil<>(MachineryJobDataVo.class);
//      util.exportExcel(response, list, "农机作业信息");
        AjaxResult excelName = util.exportExcel(list, "农机作业信息");
        String msg = (String) excelName.get("msg");
        Map<String, String> result = new HashMap<>();
        result.put("path", RuoYiConfig.getDownloadPath() + msg);
        return AjaxResult.success(result);
    }

    /**
     * 查询农机轨迹信息
     *
     * @param workplace
     * @param pttime
     * @return
     */
    @GetMapping("/VehicleTrack")
    public List<MachineryTrajectory> selectVehicleTrack(String workplace, Date pttime) {
        return iFarmMachineryService.selectVehicleTrack(workplace, pttime);
    }


    /**
     * 农机设备状态查询
     *
     * @return
     */
    @GetMapping("/selectFarmStatus")
    public FarmStatus selectFarmStatus() {
        FarmStatus farmStatus = new FarmStatus();
        //农机设备信息查询
        List<FarmMachineryVo> farmMachineryVos = iFarmMachineryService.selectMachineryData();
        int total = farmMachineryVos.size();
        farmStatus.setTotal(total);
        int working = iFarmMachineryService.selectWorkingMachinery();
        farmStatus.setWorking(working);
        farmStatus.setUnwork(total - working);
        int Online = iFarmMachineryService.selectMachineryOnline();
        farmStatus.setOnline(Online);
        farmStatus.setOffline(total - Online);
        return farmStatus;
    }

    //定时获取数据库农机轨迹并发布服务
    @Scheduled(fixedDelay = 10* 60 * 1000)
    public AjaxResult Machinert() throws Exception {
        Map<String, Map<String, String>> areaWorkhoursMap = new HashMap<>();
        //查询作业地块
        List<String> workplaceList = iFarmMachineryService.distinctWorkplace();
        Map result = new HashMap<>();
        StringBuilder multiPolygon = new StringBuilder("MULTIPOLYGON((");
        List<String> userdata = new ArrayList<>();
        StringBuilder multiLineString = new StringBuilder("MULTILINESTRING(");
        for (String workplace : workplaceList) {
            List<MyPoint> pointList = new ArrayList<>();
            Date date = new Date(2023 - 1900, 3 - 1, 13);
            //根据作业地块和时间查询农机轨迹数据
            List<MachineryTrajectory> machineryTrajectories = iFarmMachineryService.selectVehicleTrack(workplace, date);
            for (MachineryTrajectory machineryTrajectory : machineryTrajectories) {
                pointList.add(new MyPoint(machineryTrajectory.getLng(), machineryTrajectory.getLat()));
            }
            //数据大于1000进行抽稀缩减
            if (pointList.size() > 1000) {
                for (int i = 0; i < pointList.size(); i += 3) {
                    pointList.remove(i);
                }
            }

            String polygon = ShapeUtil.point2WktPolygon(pointList);
            String line = ShapeUtil.point2WktLine(pointList);
            Map<String, String> map = new HashMap<>();
            map.put("polygon", polygon);

            //在经纬度数组后加入第一个点位，形成闭环
            pointList.add(pointList.size(), pointList.get(0));
            //使用凸包算法过滤出边界
            List<MyPoint> myPoints = iMapDrawPolygonService.DrawPolygon(pointList);
            //第二次过滤转换为wkt格式
            String drawPolygon = ShapeUtil.point2WktPolygon(iMapDrawPolygonService.DrawPolygon(myPoints));
            //userdata中添加农机编号
            MachineryTrajectory trajectory = machineryTrajectories.get(0);
            //查询农机编号
            String vehicleno = trajectory.getVehicleno();
            userdata.add(vehicleno);
            userdata.add(trajectory.getWorkplace());
            //通过农机编号查询作业类型
            userdata.add(String.valueOf(iFarmMachineryService.selectJobtype(vehicleno)));
            userdata.add(DateUtils.getDate());
            //截取wkt格式字符串，改变为MULTIPOLYGON格式
            String substring = drawPolygon.substring(8, drawPolygon.length() - 1);
            String linestring = line.substring(10);
            multiPolygon.append(substring).append("),(");
            multiLineString.append(linestring).append(",");

            //计算地块面积（亩）
            String area = String.format("%.2f", ShapeUtil.getAreaByWkt(polygon) / 666.67);
            map.put("area", area);
            //计算作业时间

            float workingHours = Float.parseFloat(DateUtil.dateDiffHours(trajectory.getPttime(),
                    machineryTrajectories.get(machineryTrajectories.size() - 1).getPttime()));
            String pttime = String.valueOf(workingHours);
            map.put("workHours", pttime);
            if (areaWorkhoursMap.containsKey(trajectory.getVehicleno())) {
                Map<String, String> sMap = areaWorkhoursMap.get(trajectory.getVehicleno());
                float va = Float.parseFloat(area);
                float fa = Float.parseFloat(sMap.get("area"));
                area = String.format("%.2f", (va + fa));

                float vw = Float.parseFloat(pttime);
                float fw = Float.parseFloat(sMap.get("workHours"));
                pttime = String.format("%.2f", (vw + fw));
            }
            Map<String, String> mp = new HashMap<>();
            mp.put("area", area);
            mp.put("workHours", pttime);
            areaWorkhoursMap.put(trajectory.getVehicleno(), mp);

            result.put(workplace, map);

        }
        //统计作业时长和面积插入数据库
        for (Map.Entry<String, Map<String, String>> entry : areaWorkhoursMap.entrySet()) {
            String vehicleno = entry.getKey();
            Map<String, String> map = entry.getValue();
            double area = Float.parseFloat(map.get("area"));
            double workingHours = Float.parseFloat(map.get("workHours"));
            iFarmMachineryService.updateAreaWorkhours(vehicleno, area, workingHours);
        }

        String mPolygon = multiPolygon.delete(multiPolygon.length() - 2,multiPolygon.length()).append(")").toString();

        String mLineString = multiLineString.deleteCharAt(multiLineString.length() - 1).append(")").toString();

        String polygonPath = RuoYiConfig.getUploadPath()+ "/shpFile/" + LocalDate.now() + "polygon.shp";
        String lineStringPath = RuoYiConfig.getUploadPath()+ "/shpFile/lineString.shp";
//        生成shp文件
        ShapeUtil.writeShape(mPolygon,"MultiPolygon",polygonPath,userdata);
        ShapeUtil.writeShape(mLineString,"MultiLineString",lineStringPath,userdata) ;
        String fileName = LocalDate.now() + "polygon";
        String polygonZipPath = RuoYiConfig.getUploadPath()+ "/shpFile/" + LocalDate.now() + "polygon.zip";
        String lineStringZipPath = RuoYiConfig.getUploadPath()+ "/shpFile/lineString.zip";
        //发布shp服务
        Boolean polygonShpResult = iGeoServer.publishShp("PJZ", fileName, fileName, polygonZipPath);
        Boolean lineStringShpResult = iGeoServer.publishShp("PJZ", "lineString", "lineString", lineStringZipPath);
        if (polygonShpResult){
            GeoserverShp geoserverShp = new GeoserverShp();
            geoserverShp.setFileName(fileName);
            geoserverShp.setTime(new Date());
            iGeoserverShpService.insert(geoserverShp);
        }
        result.put("publishPolygonShp", polygonShpResult);
        result.put("publishlineStringShp", lineStringShpResult);
        return AjaxResult.success(result);
    }

    /**
     * 物联网接口获取农机轨迹数据
     *
     * @param vehicleno
     * @param jobday
     * @param jobtype
     * @return
     */
    public String gettrack(String vehicleno, Date jobday, String jobtype) {
        //格式化作业日期
        String formatjobday = new SimpleDateFormat("yyyy-MM-dd").format(jobday);
        Map<String, Object> map = new HashMap<>();//存放参数
        map.put("clienttoken", clienttoken);
        //农机编号
        map.put("vehicleno", vehicleno);
        //作业日期
        map.put("jobday", formatjobday);
        //作业类型码
        map.put("jobtype", jobtype);
        String result = HttpUtil.createPost(vehicleTrackUrl).form(map).execute().body();
        return result;
    }

    //定时获取农机轨迹信息
    @Scheduled(fixedDelay = 10 * 1000)
    public void processPositions() {
//        List<MachineryJobDataVo> machineryJobDataVos = iFarmMachineryService.selectMachineryJobData(DateUtil.getDayBegin(), DateUtil.getBeginDayOfTomorrow(), null);
        //查询作业信息
        List<MachineryJobDataVo> machineryJobDataVos = iFarmMachineryService.selectMachineryJobData();
        iFarmMachineryService.updateworkplace();
        //根据农机作业信息进行轨迹查询
        for (MachineryJobDataVo machineryJobDataVo : machineryJobDataVos) {
//            String json = gettrack(machineryJobDataVo.getVehicleno(), DateUtil.getDayBegin(), machineryJobDataVo.getJobtype());
            String json = gettrack(machineryJobDataVo.getVehicleno(), machineryJobDataVo.getJobday(), machineryJobDataVo.getJobtype());
            JSONObject jsonObject = JSONObject. parseObject(json);
            String postions = jsonObject.getString("postions");
            String vehicleno = jsonObject.getString("vehicleno");
            List<MachineryTrajectory> list = JSON.parseArray(postions, MachineryTrajectory.class);
            iFarmMachineryService.insertProcessPositions(list, vehicleno);
        }
    }

    //农机作业任务查询每天九点查询一次
    @Scheduled(cron = "0 0 9 * * ?")
    public void getMachineryJobData() throws ParseException {
        Map<String, Object> map = new HashMap<>();//存放参数
        map.put("clienttoken", clienttoken);
        //按要求格式化日期数据进行查询七天数据

        Date beginDayOf7day = DateUtil.getBeginDayOf7day();
        Date dayBegin = DateUtil.getDayBegin();
        String startTime = DateUtils.parseDateToStr("yyyy/MM/dd", beginDayOf7day);
        String endTime = DateUtils.parseDateToStr("yyyy/MM/dd", dayBegin);
        map.put("jobday", "[\"" + startTime + "\",\"" + endTime + "\"]");
        String result = HttpUtil.createPost(vehicleJobDataUrl).form(map).execute().body();
        //解析json
        JSONObject jsonObject = JSONObject.parseObject(result);
        JSONArray jobdata = jsonObject.getJSONArray("jobdata");
        //今日农机作业任务数据
        List<MachineryJobData> list = JSONArray.parseArray(jobdata.toJSONString(), MachineryJobData.class);
        //进行数据去重
        List<MachineryJobData> machineryJobData = iFarmMachineryService.selectJobDataByTime(beginDayOf7day, dayBegin);
        for (int i = 0; i < list.size(); i++) {
            MachineryJobData mj = list.get(i);
            for (MachineryJobData md : machineryJobData) {
                if (mj.getJobday().equals(md.getJobday()) &&
                        mj.getVehicleno().equals(md.getVehicleno()) &&
                        mj.getJobarea().equals(md.getJobarea())) {
                    list.remove(i);
                    i--;
                }
            }
        }
        if (!list.isEmpty()) {
            logger.info("更新农机作业任务");
            iFarmMachineryService.insertJobDataBatch(list);
        }
    }

    //每周一九点获取农机数据
    @Scheduled(cron = " 0 0 9 ? * 2")
    public void getMachineryData() {
        Map<String, Object> map = new HashMap<>();//存放参数
        map.put("clienttoken", clienttoken);
        String result = HttpUtil.createPost(vehicleListUrl).form(map).execute().body();
        Gson gson = new Gson();
        GsonFarmMachineryBean vehicleList = gson.fromJson(result, GsonFarmMachineryBean.class);
        //创建一个空的list集合存放json解析的Meteorological实体数据
        List<FarmMachinery> list = vehicleList.getVehicle();
        //从数据库里查询出来的数据
        List<FarmMachineryVo> farmMachineryList = iFarmMachineryService.selectMachineryData();
        //剔除两个集合里面重复元素根据Vmeid和Vehicleno
        for (int i = 0; i < list.size(); i++) {
            FarmMachinery fm = list.get(i);
            for (FarmMachineryVo f : farmMachineryList) {
                if (fm.getOwnername().equals(f.getOwnername()) && fm.getVehicleno().equals(f.getVehicleno())) {
                    list.remove(i);
                    i--;//不进行i--会数组下标越界异常
                }
            }
        }
        if (!list.isEmpty()) {
            logger.info("更新农机数据");
            iFarmMachineryService.insertBatch(list);
        }
    }

}
