package com.ruoyi.web.controller.crops;

import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.crops.domain.InterfaceStatistics;
import com.ruoyi.crops.domain.vo.InterfaceStatisticsVo;
import com.ruoyi.crops.service.InterfaceStatisticsService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import static com.ruoyi.common.utils.Threads.sleep;

@RestController
@RequestMapping("/interface/statistics")
public class InterfaceStatisticsController extends BaseController {

    private static final Logger logger = LoggerFactory.getLogger(InterfaceStatisticsController.class);

    @Autowired
    private RedisTemplate redisTemplate;
    @Autowired
    private InterfaceStatisticsService interfaceStatisticsService;

    @Scheduled(fixedDelay = 60 * 60 * 1000)
    public AjaxResult getStatistics(){
        //启动时延时一秒，其他定时任务先执行
        sleep(1000);
        InterfaceStatistics interfaceStatistics = (InterfaceStatistics) redisTemplate.opsForValue().get("interfaceStatistics");
        logger.info("每小时更新接口访问数据");
        return toAjax(interfaceStatisticsService.insert(interfaceStatistics));
    }
    @PreAuthorize("@ss.hasPermi('interface:statistics:day')")
    @GetMapping("/day")
    public InterfaceStatistics day(){
        return interfaceStatisticsService.selectByDay();
    }
    @PreAuthorize("@ss.hasPermi('interface:statistics:total')")
    @GetMapping("/total")
    public InterfaceStatisticsVo total(){
        return interfaceStatisticsService.selectTotal();
    }

}
