package com.ruoyi.web.controller.crops;

import com.ruoyi.common.core.controller.BaseController;

import com.ruoyi.crops.domain.MachineParameter;
import com.ruoyi.crops.domain.OperationRecords;
import com.ruoyi.crops.domain.vo.MachineParameterVo;
import com.ruoyi.crops.service.IMachineParameterService;
import com.ruoyi.crops.service.IOperationRecordsService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.lang.reflect.Field;
import java.time.Duration;
import java.util.HashMap;

import java.util.List;
import java.util.Map;
import java.util.Objects;

import static org.assertj.core.util.DateUtil.now;


/**
 * 水肥一体机参数controller
 */
@RestController
@RequestMapping("/machine/parameter")
public class    MachineParameterController extends BaseController {

    @Autowired
    private IMachineParameterService machineParameterService;

    @Autowired
    private IOperationRecordsService iOperationRecordsService;

    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 获取水肥一体机状态进行两次对比，得到哪些参数被修改，存入操作记录
     *
     * @param type
     * @return
     * @throws Exception
     */
    @GetMapping("/getcompare")
    public String getMachineParameter(Integer type) throws Exception {
        //判断是开启还是关闭界面
        if (type == 0) {
            //临时存入redis中和第二次做对比
            MachineParameter machineParameter = machineParameterService.getMachineParameter();
            redisTemplate.opsForValue().set("machineParameter", machineParameter, Duration.ofMinutes(10));
            return machineParameter.toString();
        } else if (type == 1) {
            //操作好关闭界面再次发送请求
            MachineParameter machineParameter = machineParameterService.getMachineParameter();
            MachineParameter parameter = (MachineParameter) redisTemplate.opsForValue().get("machineParameter");
            //记录修改了几个字段值
            int count = 0;
            StringBuilder sb = new StringBuilder();

            //反射获取类对象
            for (Field declaredField : MachineParameter.class.getDeclaredFields()) {
                try {
                    //通过下面compareColumns里定义的字段名进行对比
                    if (compareColumns.containsKey(declaredField.getName())) {
                        declaredField.setAccessible(true);
                        // 转成string 比较字符串内容
                        Object obj1 = declaredField.get(parameter);
                        Object obj2 = declaredField.get(machineParameter);
                        if (Objects.nonNull(obj1) && !obj1.equals(obj2)) {
                            OperationRecords operationRecords = new OperationRecords();
                            //状态为0阀门关闭
                            if (obj2.equals("0")) {
                                operationRecords.setOperationContent(compareColumns.get(declaredField.getName()) + "关闭");
                            } else if (obj2.equals("1")) {
                                operationRecords.setOperationContent(compareColumns.get(declaredField.getName()) + "打开");
                            }
                            operationRecords.setUpdateTime(now());
                            //插入操作记录
                            iOperationRecordsService.insert(operationRecords);
                            if (count == 2) {
                                sb.append("等信息已更新");
                                break;
                            }

                            if (count > 0) {
                                sb.append(",");
                            }
                            sb.append(compareColumns.get(declaredField.getName()));
                            count++;
                        }
                    }
                } catch (IllegalAccessException e) {
                    // log.info("initCustomerDetailCount:{}", e);
                }
            }

            return ("compare result:" + sb.toString());
        }
        return "参数错误";

    }

    Map<String, String> compareColumns = new HashMap<String, String>() {
        {
            put("Valve1", "阀门1");
            put("valve2", "阀门2");
            put("Valve3", "阀门3");
            put("Valve4", "阀门4");
            put("Valve5", "阀门5");
            put("Valve6", "阀门6");
            put("Valve7", "阀门7");
            put("Valve8", "阀门8");
            put("Valve9", "阀门9");
            put("Valve10", "阀门10");
            put("Valve11", "阀门11");
            put("Valve12", "阀门12");
            put("Valve13", "阀门13");
            put("Valve14", "阀门14");
            put("Valve15", "阀门15");
            put("Valve16", "阀门16");
            put("Valve17", "阀门17");
            put("Valve18", "阀门18");
            put("Valve19", "阀门19");
            put("Valve20", "阀门20");
            put("JiaoBan1", "搅拌1");
            put("JiaoBan2", "搅拌2");
            put("JiaoBan3", "搅拌3");
            put("JiaoBan4", "搅拌4");
            put("PowerMode", "功率模式");
            put("ControlMode", "控制模式");
        }
    };


    /**
     * 获取ph值，流量，压力值
     *
     * @return
     */

    @GetMapping("/select")
    public MachineParameterVo selectParameter() throws Exception {
        return machineParameterService.selectParameter();
    }

}
