package com.ruoyi.web.controller.crops;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.http.HttpUtil;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.ruoyi.common.core.domain.AjaxResult;

import com.ruoyi.crops.domain.gsonBean.GsonMeteorologicalBean;
import com.ruoyi.crops.domain.Meteorological;
import com.ruoyi.crops.service.IMeteorologicalService;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;


@RestController
@RequestMapping("/crops")
/**
 * 天气数据
 */
public class MeteorologicalController {
    private static final Logger logger = LoggerFactory.getLogger(MeteorologicalController.class);

    @Value("${Meteorological.url}")
    private String url;
    @Value("${Meteorological.key}")
    private String key;
    @Value("${Meteorological.code}")
    private String code;

    @Autowired
    private IMeteorologicalService iMeteorologicalService;


    /**
     * 查询预警数据
     *
     * @return
     */
    @PreAuthorize("@ss.hasPermi('crops:meterological:select')")
    @GetMapping("/meterological")
    public AjaxResult selectByTime() {
        if (iMeteorologicalService.selectByTime().isEmpty()) {
            return AjaxResult.warn("今日暂无数据");
        }
        return AjaxResult.success(iMeteorologicalService.selectByTime());
    }

    /**
     * 定时查询天气数据并去重保存
     */
    @Scheduled(fixedDelay = 60 * 60 * 1000)
    public List<Meteorological> MeteorologicalInfo() {
        Map<String, Object> map = new HashMap<>();//存放参数
        map.put("city_code", code);
        map.put("key", key);
        //发送get请求并接收响应数据
        String result = HttpUtil.createGet(url).form(map).execute().body();

        //格式化Json时间类型
        Gson gson = new GsonBuilder().setDateFormat("yyyy-MM-dd HH:mm:ss").create();
        //将json转换成对应实体类
        GsonMeteorologicalBean model = gson.fromJson(result, GsonMeteorologicalBean.class);
        //判断json是否有数据
        if (model.result != null) {
            List<Meteorological> list = model.result;
            //从数据库查询出今日的预警信息
            List<Meteorological> meteorologicals = iMeteorologicalService.selectByTime();
            //剔除两个集合里面重复元素根据WarnId和Time
            for (int i = 0; i < list.size(); i++) {
                Meteorological me = list.get(i);
                for (int j = 0; j < meteorologicals.size(); j++) {
                    Meteorological m = meteorologicals.get(j);
                    if (me.getId() == Integer.parseInt(m.getWarnId()) && me.getTime().equals(m.getTime())) {
                        list.remove(i);
                        i--;
                    }
                }
            }
            if (!list.isEmpty()) {
                //list顺序反转
                Collections.reverse(list);
                logger.info("每小时更新1次天气预警");
                iMeteorologicalService.insertBatch(list);
            } else {
                logger.info("本小时内无新预警");
            }

            return list;

        } else {
            logger.info("今日暂无预警");
        }
        return model.result;
    }
}
