package com.ruoyi.web.controller.crops;

import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.crops.domain.ServiceType;
import com.ruoyi.crops.service.IServiceTypeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 服务类型Controller
 */
@RestController
@RequestMapping("/service/type")
public class ServiceTypeController extends BaseController {
    @Autowired
    private IServiceTypeService typeService;

    /**
     * 根据类型查询（有参）
     *
     * @param type
     * @return
     */
    @PreAuthorize("@ss.hasPermi('crops:service:type')")
    @GetMapping("/{type}")
    public List<ServiceType> selectByType(@PathVariable String type) {
        return typeService.selectByType(type);
    }

    /**
     * 无参查询全部展示
     *
     * @return
     */
    @PreAuthorize("@ss.hasPermi('service:type:selectAll')")
    @GetMapping
    public List<ServiceType> selectAll() {
        return typeService.selectAll();
    }

    /**
     * 新增单条数据
     *
     * @param serviceType
     * @return
     */
    @PreAuthorize("@ss.hasPermi('service:type:insert')")
    @PostMapping("insert")
    public AjaxResult insert(@RequestBody ServiceType serviceType) {
        return toAjax(typeService.insert(serviceType));
    }

    /**
     * 根据数组批量删除数据
     *
     * @param ids
     * @return
     */
    @PreAuthorize("@ss.hasPermi('service:type:delete')")
    @DeleteMapping("/{ids}")
    private AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(typeService.deleteByIds(ids));
    }
}
