package com.ruoyi.web.controller.crops;

import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.ShapeUtil;
import com.ruoyi.crops.domain.ServiceType;
import com.ruoyi.crops.service.IFarmMachineryService;
import com.ruoyi.crops.service.IGeoServer;
import com.ruoyi.crops.service.IUploadService;
import com.ruoyi.crops.service.IexeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/crops")
public class UploadController {

    private String workSpace = "PJZ";
    @Value("${path.exepath}")
    private String exePath;

    private String imgDataPath;
    @Value("${path.boundaryDataPath}")
    private String boundaryDataPath;
    @Autowired
    private IUploadService iUploadService;

    @Autowired
    private IGeoServer iGeoServer;

    @Autowired
    private IexeService iexeService;

    @Autowired
    private IFarmMachineryService iFarmMachineryService;

    /**
     * 上传TIFF文件
     *
     * @param type 服务类型
     * @param file Tiff文件
     * @return 文件状态
     * @throws Exception
     */
    @PostMapping("/upload")
    public AjaxResult upload(@RequestParam("type") String type, @RequestParam("file") MultipartFile file) throws Exception {
        String uploadPath = RuoYiConfig.getUploadPath();
        //保存文件到本地并获得文件路径
        String filePath = iUploadService.upload(file, uploadPath+"/tifFile");

        //调用接口上传文件到服务器
//        Map<String, Object> map = new HashMap<>();//存放参数
        File targetFile = new File(filePath);
//        map.put("file", targetFile);
//        map.put("path", "pjz/tif");
//        String result = HttpUtil.createPost(url).form(map).execute().body();

        //获取文件信息
        String name = targetFile.getName();
        String[] n = name.split("\\.");

        String time = n[0].substring(n[0].length() - 10);
        String fileName = n[0];
        String servieName = workSpace + ":" + fileName;

        //封装成serviceType对象
        ServiceType serviceType = new ServiceType();

        serviceType.setServiceType(type);
        Date date = new SimpleDateFormat("yyyy-MM-dd").parse(time);
        serviceType.setTime(date);
        serviceType.setServiceName(servieName);
        serviceType.setFileName(name);
        serviceType.setFilePath(filePath);
        serviceType.setStyle(type);

        //发送地图服务
        String pjz = iGeoServer.publishTiff(workSpace, fileName, filePath, type);

        //调用exe批量导入数据
        String res = iexeService.execCommand(type, targetFile.getAbsolutePath(), boundaryDataPath);
        //插入数据库
        iGeoServer.insert(serviceType);
        return AjaxResult.success(pjz + res);

    }

    /**
     * 上传shp.zip文件
     * @param file
     * @return
     */
    @PostMapping("/uploadShp")
    public AjaxResult uploadShp(@RequestParam("file") MultipartFile file){
        String uploadPath = RuoYiConfig.getUploadPath();
        //保存文件到本地并获得文件路径
        String filePath = null;
        try {
            filePath = iUploadService.upload(file, uploadPath+"/shpFile");
            //解压上传的shp.zip文件
            String unzip = ShapeUtil.unzip(filePath, uploadPath + "/shpFile/");
            return AjaxResult.success(unzip);

        } catch (IOException e) {
            e.printStackTrace();
            return AjaxResult.error(e.getMessage());
        }
    }
}
