package com.ruoyi.web.controller.crops;

import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.utils.DateUtil;
import com.ruoyi.crops.domain.DeviceSense;
import com.ruoyi.crops.domain.WaringNumber;
import com.ruoyi.crops.domain.WarningInformation;
import com.ruoyi.crops.service.IDeviceSenseService;
import com.ruoyi.crops.service.IWarningService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

/**
 * 预警信息Controller
 */
@RestController
@RequestMapping("/warning/information")
public class WarningController extends BaseController {
    @Autowired
    private IWarningService iWarningService;

    /**
     * 预警信息根据时间查询
     *
     * @return
     */
    @PreAuthorize("@ss.hasPermi('warning:information:selectByTime')")
    @GetMapping("/selectByTime")
    public TableDataInfo selectBytTime(@RequestParam(required =  false) Date startTime,
                                       @RequestParam(required =  false) Date endTime) {
        startPage();
        List<WarningInformation> list = iWarningService.selectByTime(startTime, endTime);
        return getDataTable(list);
    }

    /**
     * 新增单条数据
     *
     * @param warningInformation
     * @return
     */
    @PostMapping
    public AjaxResult insert(@RequestBody WarningInformation warningInformation) {
        return toAjax(iWarningService.insert(warningInformation));
    }

    /**
     * 大棚预警次数查询
     *
     * @return
     */
    @PreAuthorize("@ss.hasPermi('warning:information:numberOfTimes')")
    @GetMapping("/numberOfTimes")
    public AjaxResult numberOfTimes() {
        List<WarningInformation> daylist = iWarningService.selectByTime(DateUtil.getDayBegin(), DateUtil.getDayEnd());
        int dayNumber = daylist.size();

        List<WarningInformation> weeklist = iWarningService.selectByTime(DateUtil.getBeginDayOfWeek(), DateUtil.getEndDayOfWeek());
        int weekNumber = weeklist.size();

        List<WarningInformation> monthlist = iWarningService.selectByTime(DateUtil.getBeginDayOfMonth(), DateUtil.getEndDayOfMonth());
        int monthNumber = monthlist.size();
        WaringNumber waringNumber = new WaringNumber();
        waringNumber.setDayNumber(dayNumber);
        waringNumber.setWeekNumber(weekNumber);
        waringNumber.setMonthNumber(monthNumber);
        return AjaxResult.success(waringNumber);
    }

}

