package com.ruoyi.web.controller.crops;

import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.utils.DateUtil;
import com.ruoyi.crops.domain.MachineryTrajectory;
import com.ruoyi.crops.domain.WaringNumber;
import com.ruoyi.crops.domain.WarningFarm;
import com.ruoyi.crops.domain.vo.FarmMachineryVo;
import com.ruoyi.crops.service.IFarmMachineryService;
import com.ruoyi.crops.service.IWarningFarmService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

@RestController
@RequestMapping("/warning/farm")
public class WarningFarmController extends BaseController {
    @Autowired
    private IFarmMachineryService iFarmMachineryService;
    @Autowired
    private IWarningFarmService iWarningFarmService;

    private static final Logger logger = LoggerFactory.getLogger(WarningFarmController.class);

    /**
     * 农机预警次数查询
     * @return
     */
    @GetMapping("/numberOfTimes")
    public AjaxResult numberOfTimes() {
        List<WarningFarm> daylist = iWarningFarmService.selectByTime(DateUtil.getDayBegin(), DateUtil.getDayEnd());
        int dayNumber = daylist.size();

        List<WarningFarm> weeklist = iWarningFarmService.selectByTime(DateUtil.getBeginDayOfWeek(), DateUtil.getEndDayOfWeek());
        int weekNumber = weeklist.size();

        List<WarningFarm> monthlist = iWarningFarmService.selectByTime(DateUtil.getBeginDayOfMonth(), DateUtil.getEndDayOfMonth());
        int monthNumber = monthlist.size();
        WaringNumber waringNumber = new WaringNumber();
        waringNumber.setDayNumber(dayNumber);
        waringNumber.setWeekNumber(weekNumber);
        waringNumber.setMonthNumber(monthNumber);
        return AjaxResult.success(waringNumber);
    }

    /**
     * 农机预警信息根据时间查询
     * @param startTime
     * @param endTime
     * @return
     */
    @GetMapping("/selectByTime")
    public TableDataInfo selectBytTime(@RequestParam(required =  false) Date startTime,
                                       @RequestParam(required =  false) Date endTime) {
        startPage();
        List<WarningFarm> list = iWarningFarmService.selectByTime(startTime, endTime);
        return getDataTable(list);
    }

    //判断农机预警
    @Scheduled(fixedDelay = 5 * 60 * 1000)
    public void warning() {
        //查询今日作业农机编号
        List<String> vehiclenoList = iFarmMachineryService.distinctVehicleno();
        //根据编号查询作业地块
        for (String vehicleno : vehiclenoList) {
            List<String> workplaceList = iFarmMachineryService.distinctWorkplace(vehicleno);
            Date lastTime = null;
            //根据地块查询具体作业数据
            for (String workplace : workplaceList) {
                Date date = new Date(2023 - 1900, 3 - 1, 13);
                //根据作业地块和时间查询农机轨迹数据
                List<MachineryTrajectory> machineryTrajectories = iFarmMachineryService.selectVehicleTrack(workplace, date);
                MachineryTrajectory trajectory = machineryTrajectories.get(0);
                Date lastPttime = machineryTrajectories.get(machineryTrajectories.size() - 1).getPttime();
                //农机此次作业时长
                float workingHours = Float.parseFloat(DateUtil.dateDiffHours(trajectory.getPttime(), lastPttime));

                WarningFarm warningFarm = new WarningFarm();
                warningFarm.setVehicleno(vehicleno);
                warningFarm.setTime(new Date());
                FarmMachineryVo farmMachineryVos = iFarmMachineryService.selectMachineryData(vehicleno);
                warningFarm.setOwnername(farmMachineryVos.getOwnername());
                if (lastTime != null) {
                    float freeTime = Float.parseFloat(DateUtil.dateDiffHours(lastTime, trajectory.getPttime()));
                    if (freeTime > 0.1) {
                        //插入数据库预警
                        warningFarm.setWarningInfo("闲置时间过长");
                        iWarningFarmService.insertWarning(warningFarm);
                        logger.info("农机闲置时间预警更新：" + vehicleno +"-"+ freeTime + "小时");
                    }
                }
                lastTime = machineryTrajectories.get(machineryTrajectories.size() - 1).getPttime();
                if (workingHours > 0.5) {
                    //预警
                    warningFarm.setWarningInfo("作业时间过长");
                    iWarningFarmService.insertWarning(warningFarm);
                    logger.info("农机作业时间预警更新：" + vehicleno +"-"+ workingHours + "小时");
                }
            }
        }
    }

}
