package com.ruoyi.web.controller.crops;

import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.crops.domain.WeatherPredictionEntity;
import com.ruoyi.crops.domain.vo.WeatherPredoctionVo;
import com.ruoyi.crops.service.IWeatherPredictionService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.List;

@RestController
@RequestMapping("/crops/weather")
public class WeatherPredictionController extends BaseController {

    @Autowired
    private IWeatherPredictionService iWeatherPredictionService;

    private static final Logger logger = LoggerFactory.getLogger(WeatherPredictionController.class);

    /**
     * 查询一周的天气数据
     *
     * @return
     */
    @PreAuthorize("@ss.hasPermi('crops:weather:selectWeek')")
    @GetMapping("/selectWeek")
    public List<WeatherPredictionEntity> selectWeek() {
        return iWeatherPredictionService.selectWeek();
    }


    /**
     * 气象数据对比查询
     *
     * @param time1
     * @param time2
     * @return
     */
    @PreAuthorize("@ss.hasPermi('crops:weather:compareTime')")
    @GetMapping("/compareTime")
    public List<WeatherPredoctionVo> compareTime(@RequestParam Date time1, @RequestParam Date time2) {
        List<WeatherPredoctionVo> weather1 = iWeatherPredictionService.selectByTime(time1);
        List<WeatherPredoctionVo> weather2 = iWeatherPredictionService.selectByTime(time2);
        for (WeatherPredoctionVo weatherPredoctopnVo : weather1) {
            if (weatherPredoctopnVo != null) {
                weatherPredoctopnVo.setWeatherTime(time1);
                String windSpeed = String.format("%.2f", Double.parseDouble(weatherPredoctopnVo.getWindSpeed()));
                String precipitation = String.format("%.2f", Double.parseDouble(weatherPredoctopnVo.getPrecipitation()));
                weatherPredoctopnVo.setWindSpeed(windSpeed);
                weatherPredoctopnVo.setPrecipitation(precipitation);
            }
        }
        for (WeatherPredoctionVo weatherPredoctopnVo : weather2) {
            if (weatherPredoctopnVo != null) {
                weatherPredoctopnVo.setWeatherTime(time2);
                String windSpeed = String.format("%.2f", Double.parseDouble(weatherPredoctopnVo.getWindSpeed()));
                String precipitation = String.format("%.2f", Double.parseDouble(weatherPredoctopnVo.getPrecipitation()));
                weatherPredoctopnVo.setWindSpeed(windSpeed);
                weatherPredoctopnVo.setPrecipitation(precipitation);
            }
        }
        List<WeatherPredoctionVo> list = new ArrayList<>();
        list.addAll(weather1);
        list.addAll(weather2);
        return list;

    }

    /**
     * 定时查询更新天气数据
     *
     * @throws IOException
     */
    //    @Scheduled(cron = "0 0 8,12,20 * * ?")
    @Scheduled(fixedDelay = 60 * 60 * 1000)
    public List<WeatherPredictionEntity> getWeather() throws IOException {
        List<WeatherPredictionEntity> weatherList = iWeatherPredictionService.getWeather();
        iWeatherPredictionService.insertBatch(weatherList);
        logger.info("每小时更新博兴天气数据");
        return weatherList;
    }
}
