package com.ruoyi.crops.service;

import com.ruoyi.crops.domain.FarmMachinery;
import com.ruoyi.crops.domain.MachineryJobData;
import com.ruoyi.crops.domain.MachineryPosition;
import com.ruoyi.crops.domain.MachineryTrajectory;
import com.ruoyi.crops.domain.vo.FarmMachineryVo;
import com.ruoyi.crops.domain.vo.MachineryJobDataVo;
import org.apache.ibatis.annotations.Param;
import org.locationtech.jts.geom.Coordinate;

import java.util.Date;
import java.util.List;

public interface IFarmMachineryService {

    /**
     * 将字符串多坐标点转换为坐标数组
     *
     * @param latLonString "116.616634,40.272665|116.644733,40.280371|116.636181,40.264352|116.616634,40.272665"
     * @return org.locationtech.jts.geom.Coordinate[]
     */
    public Coordinate[] getCoordinateArray(String latLonString);

    /**
     * 判断坐标点是否在多坐标点组成的多边形面内
     *
     * @param coordinateArray
     * @param coordinate
     * @return boolean
     * @author guochao.bj@fang.com
     * within 判断是否在内部，边缘点返回false
     * intersects 判断是否相交，弥补 within边缘点缺陷，
     */
    public boolean withinAndIntersects(Coordinate[] coordinateArray, Coordinate coordinate);

    /**
     * @param lnglatPolygon 坐标数组
     * @param lng           经度
     * @param lat           纬度
     * @return
     */
    public boolean withinAndIntersects(String lnglatPolygon, double lng, double lat);

    /**
     * 农机信息查询
     * @return
     */
    public List<FarmMachineryVo> selectMachineryData();

    public FarmMachineryVo selectMachineryData(String vehicleno);

    /**
     * 在线农机数量查询
     * @return
     */
    public int selectMachineryOnline();

    /**
     * 批量插入农机设备信息
     * @param farmMachineryList
     * @return
     */
    public int insertBatch(List<FarmMachinery> farmMachineryList);

    /**
     * 批量插入农机作业任务
     * @param list
     * @return
     */
    public int insertJobDataBatch(List<MachineryJobData> list);

    /**
     * 农机作业任务查询
     * @param startTime
     * @param endTime
     * @return
     */
    public List<MachineryJobData> selectJobDataByTime(Date startTime,Date endTime);

    /**
     * 历史作业信息查询
     * @param startTime
     * @param endTime
     * @param vehicleno
     * @return
     */
    List<MachineryJobDataVo> selectMachineryJobData(Date startTime,Date endTime,String vehicleno);

    List<MachineryJobDataVo> selectMachineryJobData();

    /**
     * 农机轨迹数据分类
     * @param machineryTrajectories
     * @param vehicleno
     * @return
     */
    public List<List<MachineryTrajectory>> processPositions(List<MachineryTrajectory> machineryTrajectories,String vehicleno);

    /**
     * 插入农机轨迹数据
     * @param list
     * @param vehicleno
     * @return
     */
    public void insertProcessPositions(List<MachineryTrajectory> list,String vehicleno);

    /**
     * 查询农机轨迹数据
     * @param workplace
     * @param pttime
     * @return
     */
    List<MachineryTrajectory> selectVehicleTrack(String workplace, Date pttime);

    /**
     * 查询今日作业地块
     * @param vehicleno 农机编号（为空查询今日全部）
     * @return
     */
    List<String> distinctWorkplace(String vehicleno);

    List<String> distinctWorkplace();


    /**
     * 查询今日作业农机编号
     * @return
     */
    List<String> distinctVehicleno();

    /**
     * 更新作业面积和时长
     * @param vehicleno
     * @param area
     * @param workingHours
     */
    void updateAreaWorkhours(String vehicleno,double area, double workingHours);

    /**
     * 查询今日作业数据
     * @return
     */
    MachineryJobDataVo selectTodayJobData();

    /**
     * 查询正在作业农机数量
     */
    int selectWorkingMachinery();

    /**
     * 插入农机位置
     * @param position
     */
    void insertPositions(List<MachineryPosition> position);

    /**
     * 查询农机作业类型
     * @param vehicleno
     * @return
     */
    int selectJobtype(@Param("vehicleno")String vehicleno);

    void updateworkplace();

    /**
     * 查询全部农机编号
     * @return
     */
    List<String> selectVehicleno();
}

