package com.ruoyi.crops.service.impl;

import com.ruoyi.crops.domain.FarmMachinery;
import com.ruoyi.crops.domain.MachineryJobData;
import com.ruoyi.crops.domain.MachineryPosition;
import com.ruoyi.crops.domain.MachineryTrajectory;
import com.ruoyi.crops.domain.vo.FarmMachineryVo;
import com.ruoyi.crops.domain.vo.MachineryJobDataVo;
import com.ruoyi.crops.mapper.FarmMachineryMapper;
import com.ruoyi.crops.mapper.MachineryJobDataMapper;
import com.ruoyi.crops.mapper.MachineryPositionMapper;
import com.ruoyi.crops.mapper.MachineryTrajectoryMapper;
import com.ruoyi.crops.service.IFarmMachineryService;
import org.locationtech.jts.geom.Coordinate;
import org.locationtech.jts.geom.GeometryFactory;
import org.locationtech.jts.geom.Point;
import org.locationtech.jts.geom.Polygon;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class FarmMachineryServiceImpl implements IFarmMachineryService {
    //    private int lastProcessedIndex = -1;
    private List<MachineryTrajectory> previousGroup = null;
    private static int workplace;
    @Autowired
    private MachineryTrajectoryMapper machineryTrajectoryMapper;

    @Autowired
    private FarmMachineryMapper farmMachineryMapper;

    @Autowired
    private MachineryPositionMapper machineryPositionMapper;

    @Autowired
    private MachineryJobDataMapper machineryJobDataMapper;

    @Override
    public void updateworkplace(){
        workplace=64;
    }

    /**
     * 查询全部农机编号
     * @return
     */
    @Override
    public List<String> selectVehicleno() {
        return machineryJobDataMapper.selectVehicleno();
    }

    /**
     * 将字符串多坐标点转换为坐标数组
     *
     * @param latLonString "116.616634,40.272665|116.644733,40.280371|116.636181,40.264352|116.616634,40.272665"
     * @return org.locationtech.jts.geom.Coordinate[]
     */
    @Override
    public Coordinate[] getCoordinateArray(String latLonString) {
        String[] split = latLonString.split("\\|");
        Coordinate[] coordinateArray = new Coordinate[split.length];
        for (int i = 0; i < split.length; i++) {
            String[] LatLng = split[i].split(",");
            Coordinate coordinate = new Coordinate(Double.valueOf(LatLng[0]), Double.valueOf(LatLng[1]));
            coordinateArray[i] = coordinate;
        }
        return coordinateArray;
    }

    /**
     * 判断坐标点是否在多坐标点组成的多边形面内
     *
     * @param coordinateArray
     * @param coordinate
     * @return boolean
     * @author guochao.bj@fang.com
     * within 判断是否在内部，边缘点返回false
     * intersects 判断是否相交，弥补 within边缘点缺陷，
     */
    @Override
    public boolean withinAndIntersects(Coordinate[] coordinateArray, Coordinate coordinate) {
        boolean result = false;
        //小于3个点无法组成多边形
        if (coordinateArray.length < 3) {
            return result;
        }

        //必须首尾坐标点相同组成闭合多边形
        if (!(coordinateArray[0].equals2D(coordinateArray[coordinateArray.length - 1]))) {
            return result;
        }

        GeometryFactory geometryFactory = new GeometryFactory();
        Point point = geometryFactory.createPoint(coordinate);
        Polygon polygon = geometryFactory.createPolygon(coordinateArray);

        if (point.within(polygon) || point.intersects(polygon)) {
            result = true;
        }

        return result;
    }

    /**
     * @param lnglatPolygon 坐标数组
     * @param lng           经度
     * @param lat           纬度
     * @return
     */
    @Override
    public boolean withinAndIntersects(String lnglatPolygon, double lng, double lat) {
        Coordinate[] latLonMap = getCoordinateArray(lnglatPolygon);
        Coordinate coordinate = new Coordinate(lng, lat);
        boolean within = withinAndIntersects(latLonMap, coordinate);
        return within;
    }

    /**
     * 农机设备信息查询
     *
     * @return
     */
    @Override
    public List<FarmMachineryVo> selectMachineryData() {
        return farmMachineryMapper.selectMachineryData();
    }


    @Override
    public FarmMachineryVo selectMachineryData(String vehicleno) {
        return farmMachineryMapper.selectMachineryData(vehicleno);
    }

    /**
     * 查询农机在线数量
     *
     * @return
     */
    @Override
    public int selectMachineryOnline() {
        return machineryPositionMapper.selectMachineryOnline();
    }

    /**
     * 批量插入农机设备信息
     *
     * @param farmMachineryList
     * @return
     */
    @Override
    public int insertBatch(List<FarmMachinery> farmMachineryList) {
        return farmMachineryMapper.insertBatch(farmMachineryList);
    }

    /**
     * 批量插入农机作业任务
     *
     * @param list
     * @return
     */
    @Override
    public int insertJobDataBatch(List<MachineryJobData> list) {
        return machineryJobDataMapper.insertBatch(list);
    }

    /**
     * 农机作业任务查询
     *
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return
     */
    @Override
    public List<MachineryJobData> selectJobDataByTime(Date startTime, Date endTime) {
        return machineryJobDataMapper.selectByTime(startTime, endTime);
    }

    /**
     * 历史作业信息查询
     *
     * @param startTime
     * @param endTime
     * @param vehicleno
     * @return
     */
    @Override
    public List<MachineryJobDataVo> selectMachineryJobData(Date startTime, Date endTime, String vehicleno) {
        return machineryJobDataMapper.selectMachineryJobData(startTime, endTime, vehicleno);
    }
    @Override
    public List<MachineryJobDataVo> selectMachineryJobData() {
        return machineryJobDataMapper.selectMachineryJobData();
    }

    /**
     * 农机轨迹数据分类
     *
     * @param machineryTrajectories
     * @param vehicleno
     * @return
     */
    @Override
    public List<List<MachineryTrajectory>> processPositions(List<MachineryTrajectory> machineryTrajectories, String vehicleno) {
        List<List<MachineryTrajectory>> result = new ArrayList<>();
        List<MachineryTrajectory> currentGroup = null;

        boolean isPreviousWork = false;
        //判断第一次分组后结果，如果上次分组不为空并且等于1那么就和上次查询分组放在一起
        if (previousGroup != null && !previousGroup.isEmpty() && previousGroup.get(previousGroup.size() - 1).getIswork() == 1) {
            isPreviousWork = true;
            currentGroup = previousGroup;
        }

        for (int i = 0; i < machineryTrajectories.size(); i++) {
            MachineryTrajectory machineryTrajectory = machineryTrajectories.get(i);
            if (machineryTrajectory.getIswork() == 1 && machineryTrajectory.getCourse() != 0) {
                if (!isPreviousWork) {
                    //记录工作地块
                    workplace++;
                    currentGroup = new ArrayList<>();
                    result.add(currentGroup);
                }
                machineryTrajectory.setWorkplace(String.valueOf((char) workplace));
                machineryTrajectory.setVehicleno(vehicleno);
                currentGroup.add(machineryTrajectory);
                isPreviousWork = true;
            } else {
                isPreviousWork = false;
                currentGroup = null;
            }
//            lastProcessedIndex = i;
        }

        if (currentGroup != null && !currentGroup.isEmpty() && currentGroup.get(currentGroup.size() - 1).getIswork() == 1) {
            previousGroup = currentGroup;
        } else {
            previousGroup = null;
        }
        return result;
    }

    /**
     * 插入作业轨迹数据
     * @param list
     * @param vehicleno
     */
    @Override
    public void insertProcessPositions(List<MachineryTrajectory> list, String vehicleno) {
        //对轨迹数据进行分组
        List<List<MachineryTrajectory>> result = processPositions(list, vehicleno);

        for (List<MachineryTrajectory> machineryTrajectories : result) {
            List<MachineryTrajectory> res = new ArrayList<>();
            //数据抽稀每隔十条数据取出一条
            for (int i = 0; i < machineryTrajectories.size(); i += 10) {
                res.add(machineryTrajectories.get(i));
            }
            //点位小于3过滤掉
            if (res.size() <= 3) {
                continue;
            }
            //判断数据库里重复数据
            List<MachineryTrajectory> mt = machineryTrajectoryMapper
                    .selectByVehicleno(machineryTrajectories.get(machineryTrajectories.size() - 1).getVehicleno());
            for (int j = 0; j < res.size(); j++) {
                MachineryTrajectory machineryTrajectory = res.get(j);
                for (MachineryTrajectory trajectory : mt) {
                    if (machineryTrajectory.getLng() == trajectory.getLng() && machineryTrajectory.getLat() == trajectory.getLat()) {
                        res.remove(j);
                        j--;
                    }
                }
            }
            //将结果插入数据库
            if (!res.isEmpty()) {
                machineryTrajectoryMapper.insertTrajectory(res);
            }
        }
    }

    /**
     * 查询农机轨迹数据
     * @param workplace
     * @param pttime
     * @return
     */
    @Override
    public List<MachineryTrajectory> selectVehicleTrack(String workplace, Date pttime) {
        return machineryTrajectoryMapper.selectVehicleTrack(workplace, pttime);
    }

    /**
     * 查询今日作业地块
     * @param vehicleno 农机编号（为空查询今日全部）
     * @return
     */
    @Override
    public List<String> distinctWorkplace(String vehicleno) {
        return machineryTrajectoryMapper.distinctWorkplace(vehicleno);
    }

    @Override
    public List<String> distinctWorkplace() {
        return machineryTrajectoryMapper.distinctWorkplace();
    }

    /**
     * 查询今日作业农机编号
     *
     * @return
     */
    @Override
    public List<String> distinctVehicleno() {
        return machineryTrajectoryMapper.distinctVehicleno();
    }

    /**
     * 更新农机作业面积，作业时间
     *
     * @param area
     * @param workingHours
     * @param vehicleno
     */
    @Override
    public void updateAreaWorkhours(String vehicleno, double area, double workingHours) {
        machineryJobDataMapper.updateAreaWorkhours(vehicleno, area, workingHours);

    }

    /**
     * 查询今日作业数据
     * @return
     */
    @Override
    public MachineryJobDataVo selectTodayJobData() {
        return machineryJobDataMapper.selectTodayJobData();

    }

    /**
     * 查询正在工作农机数量
     * @return
     */
    @Override
    public int selectWorkingMachinery() {
        return machineryTrajectoryMapper.selectWorkingMachinery();
    }

    /**
     * 插入农机位置
     * @param position
     */
    @Override
    public void insertPositions(List<MachineryPosition> position) {
        machineryPositionMapper.insertBatch(position);
    }

    /**
     * 查询农机作业类型
     * @param vehicleno
     * @return
     */
    @Override
    public int selectJobtype(String vehicleno) {
        return machineryPositionMapper.selectJobtype(vehicleno);
    }


}
