package com.ruoyi.crops.service.impl;

import com.ruoyi.crops.domain.ServiceType;
import com.ruoyi.crops.mapper.ServiceTypeMapper;
import com.ruoyi.crops.service.IGeoServer;
import it.geosolutions.geoserver.rest.GeoServerRESTManager;
import it.geosolutions.geoserver.rest.GeoServerRESTPublisher;
import it.geosolutions.geoserver.rest.GeoServerRESTReader;
import it.geosolutions.geoserver.rest.decoder.RESTDataStore;
import it.geosolutions.geoserver.rest.encoder.GSResourceEncoder;
import it.geosolutions.geoserver.rest.encoder.datastore.GSGeoTIFFDatastoreEncoder;
import it.geosolutions.geoserver.rest.manager.GeoServerRESTStyleManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.File;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class GeoServerImpl implements IGeoServer {
    //GeoServer的连接配置
    @Value("${geoserver.url}")
    private String url;
    @Value("${geoserver.username}")
    private String username;
    @Value("${geoserver.password}")
    private String password;
    private static final Logger logger = LoggerFactory.getLogger(GeoServerImpl.class);
    @Autowired
    private ServiceTypeMapper serviceTypeMapper;

    @Override
    public String publishTiff(String workSpace, String storeName, String filePath, String styleName) throws Exception {
        String result="";
        //判断工作区（workspace）是否存在，不存在则创建
        URL u = new URL(url);
        GeoServerRESTManager manager = new GeoServerRESTManager(u, username, password);
        GeoServerRESTPublisher publisher = manager.getPublisher();
        List<String> workspaces = manager.getReader().getWorkspaceNames();
        if (!workspaces.contains(workSpace)) {
            boolean createws = publisher.createWorkspace(workSpace);
            logger.info("create ws : " + createws);
        }
        try {
            //判断数据存储（datastore）是否已经存在，不存在则创建
            RESTDataStore restStore = manager.getReader().getDatastore(workSpace, storeName);
            if (restStore == null) {
                GSGeoTIFFDatastoreEncoder gsGeoTIFFDatastoreEncoder = new GSGeoTIFFDatastoreEncoder(storeName);
                gsGeoTIFFDatastoreEncoder.setWorkspaceName(workSpace);
                gsGeoTIFFDatastoreEncoder.setUrl(new URL("file:" + filePath));
                boolean createStore = manager.getStoreManager().create(workSpace, gsGeoTIFFDatastoreEncoder);
                logger.info("create store (TIFF文件创建状态) : " + createStore);

                boolean publish = manager.getPublisher().publishGeoTIFF(workSpace, storeName, storeName,new File(filePath),"EPSG:4326",
                        GSResourceEncoder.ProjectionPolicy.FORCE_DECLARED,styleName,null);
                logger.info("publish (TIFF文件发布状态) : " + publish);

                result = "create store (TIFF文件创建状态) : " + createStore + " publish (TIFF文件发布状态) : " + publish;
            } else {
                result = "数据存储已经存在了,store:" + storeName;
            }
        }catch (Exception e){
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public Boolean publishShp(String workSpace, String storeName, String layername, String filePath) {
        boolean result = false;
        try {
            File zipFile=new File(filePath);
            GeoServerRESTReader reader = new GeoServerRESTReader(url, username, password);
                GeoServerRESTPublisher publisher = new GeoServerRESTPublisher(url, username, password);
            List<String> workspaces=reader.getWorkspaceNames();

            // 判断工作空间和图层是否存在
            boolean existsStore = reader.existsDatastore(workSpace, storeName);
            boolean existsLayer = reader.existsLayer(workSpace, layername);
            if (existsStore||existsLayer){
                logger.info("工作空间存在："+existsStore+"图层存在："+existsLayer);
                boolean removeLayer = publisher.removeLayer(workSpace, layername);
                boolean removeStore = publisher.removeDatastore(workSpace, storeName, true);
                logger.info("删除工作空间："+removeStore+"删除图层："+removeLayer);
            }

            if(workspaces.contains(workSpace)){
                if(publisher.publishShp(workSpace, storeName, layername, zipFile, "EPSG:4326")){
//                    result = "发布成功!";
                    result = true;
                }

            }
        }catch (Exception e){
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public int insert(ServiceType serviceType) {
        return serviceTypeMapper.insert(serviceType);
    }
}
