package com.ruoyi.crops.service.impl;

import com.ruoyi.crops.domain.WeatherPredictionEntity;
import com.ruoyi.crops.domain.vo.WeatherPredoctionVo;
import com.ruoyi.crops.mapper.WeatherPredictionMapper;
import com.ruoyi.crops.service.IWeatherPredictionService;
import org.jsoup.Connection;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.nodes.Node;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

@Service
public class IWeatherPredictionServiceImpl implements IWeatherPredictionService {
    @Autowired
    private WeatherPredictionMapper weatherPredictionMapper;

    @Override
    public List<WeatherPredictionEntity> getWeather() throws IOException {
        // 目标网页中国气象局
        String testURL = "https://weather.cma.cn/web/weather/54737.html";
        Connection con = Jsoup.connect(testURL);
        // 选择发送方式,获取整个网页信息，存在documnet类里
        Document document = con.get();
        // 定义天气日期列表
        List<LocalDate> weatherDayList = new ArrayList<>();
        // 定义天气名称列表
        List<String> weatherNameList = new ArrayList<>();
        // 获取当前日期
        LocalDate dateNow = LocalDate.now();
        DateTimeFormatter dfDate = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        LocalDate lastDate = dateNow.minusDays(1);
        ;
        DateTimeFormatter dfDateTime = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm");
        // 获取日期列表
        List<Node> dayList = Objects.requireNonNull(document.body().getElementById("dayList")).childNodes();

        for (Node node : dayList) {
            if (node != null && !node.toString().equals("")) {
                String datStr = dateNow.getYear() + "-" + node.childNode(1).childNode(2).toString().
                        replaceAll("\r|\n", "").replaceAll("/", "-");
                String weatherName = node.childNode(5).childNode(0).toString().replaceAll("\r|\n", "");

                LocalDate dateDay = LocalDate.parse(datStr, dfDate);
                // 如果日期未按顺序排列，则发生跨年,年份加一
                if (lastDate.isAfter(dateDay)) {
                    dateDay = dateDay.plusYears(1);
                }
                lastDate = dateDay;
                weatherDayList.add(dateDay);
                weatherNameList.add(weatherName);
            }
        }
        // 定义天气变量列表
        List<LocalDateTime> weatherTimeList = new ArrayList<>();
        List<String> airTemperatureList = new ArrayList<>();
        List<String> precipitationList = new ArrayList<>();
        List<String> windSpeedList = new ArrayList<>();
        List<String> windDirectionList = new ArrayList<>();
        List<String> atmosphericPressureList = new ArrayList<>();
        List<String> humidityList = new ArrayList<>();
        List<String> cloudCoverList = new ArrayList<>();
        // 获取分小时气象信息
        List<Element> hourTableList = new ArrayList<>(Arrays.asList(document.body().getElementById("hourTable_0"),
                document.body().getElementById("hourTable_1"), document.body().getElementById("hourTable_2"),
                document.body().getElementById("hourTable_3"), document.body().getElementById("hourTable_4"),
                document.body().getElementById("hourTable_5"), document.body().getElementById("hourTable_6")
        ));
        // 获取当前时间
        LocalDateTime dateTimeNow = LocalDateTime.now();
        int m = 0;
        // 获取逐天分小时气象信息表格数据
        for (Element elementList : hourTableList) {
            List<Node> nodeList = elementList.childNode(1).childNodes();
            LocalDateTime lastDateTime = dateTimeNow.minusMinutes(180);
            int n = 0;
            for (Node node : nodeList) {
                if (node != null && !node.toString().equals("")) {
                    List<Node> nodeList2 = node.childNodes();
                    for (Node node2 : nodeList2) {
                        if (node2 != null && !node2.toString().equals("")) {
                            n = n + 1;
                            //  如果多于一个节点
                            if (node2.childNodes().size() > 1) {
                                List<?> list = new ArrayList<>();
                                list = new ArrayList<>(new LinkedHashSet<>(list));
                            } else {
                                // 如果是单一节点,获取该节点的值
                                if (n > 1 && n < 10) {
                                    String dateTimeStr = weatherDayList.get(m) + " " + node2.childNode(0);
                                    LocalDateTime localDateTime = LocalDateTime.parse(dateTimeStr, dfDateTime);
                                    // 若跨日，日期加一
                                    if (lastDateTime.isAfter(localDateTime)) {
                                        localDateTime = localDateTime.plusDays(1);
                                    }
                                    weatherTimeList.add(localDateTime);
                                    lastDateTime = localDateTime;
                                }
                                if (n > 10 && n < 19) {
                                    weatherNameList.add(node2.childNode(0).toString());
                                }
                                if (n > 19 && n < 28) {
                                    airTemperatureList.add(node2.childNode(0).toString());
                                }
                                if (n > 28 && n < 37) {
                                    precipitationList.add(node2.childNode(0).toString());
                                }
                                if (n > 37 && n < 46) {
                                    windSpeedList.add(node2.childNode(0).toString());
                                }

                                if (n > 46 && n < 55) {
                                    windDirectionList.add(node2.childNode(0).toString());
                                }

                                if (n > 55 && n < 64) {
                                    atmosphericPressureList.add(node2.childNode(0).toString());
                                }

                                if (n > 64 && n < 73) {
                                    humidityList.add(node2.childNode(0).toString());
                                }

                                if (n > 73 && n < 82) {
                                    cloudCoverList.add(node2.childNode(0).toString());
                                }
                            }
                        }
                    }
                }
            }
            m = m + 1;
        }

        List<WeatherPredictionEntity> weatherPredictionEntityArrayList = new ArrayList<>();
        // 长度固定 8*7 = 56
        int len = 8 * 7;
        for (int i = 0; i < len; i++) {
            WeatherPredictionEntity weatherPredictionEntity = new WeatherPredictionEntity();
            weatherPredictionEntity.setWeatherName(weatherNameList.get(i / 8));
            weatherPredictionEntity.setWeatherTime(weatherTimeList.get(i));
            weatherPredictionEntity.setAirTemperature(airTemperatureList.get(i));
            weatherPredictionEntity.setPrecipitation(precipitationList.get(i));
            weatherPredictionEntity.setWindSpeed(windSpeedList.get(i));
            weatherPredictionEntity.setWindDirection(windDirectionList.get(i));
            weatherPredictionEntity.setAtmosphericPressure(atmosphericPressureList.get(i));
            weatherPredictionEntity.setHumidity(humidityList.get(i));
            weatherPredictionEntity.setCloudCover(cloudCoverList.get(i));
            weatherPredictionEntityArrayList.add(weatherPredictionEntity);
        }
        return weatherPredictionEntityArrayList;
    }

    @Override
    public int insert(WeatherPredictionEntity weatherPredictionEntity) {

        return weatherPredictionMapper.insert(weatherPredictionEntity);
    }

    @Override
    public int insertBatch(List<WeatherPredictionEntity> list) {
        return weatherPredictionMapper.insertBatch(list);
    }

    @Override
    public List<WeatherPredictionEntity> selectWeek() {
        return weatherPredictionMapper.selectWeek();
    }

    @Override
    public List<WeatherPredoctionVo> selectByTime(Date time) {
        return weatherPredictionMapper.selectByTime(time);
    }

    @Override
    public List<WeatherPredictionEntity> select(Date time) {
        return weatherPredictionMapper.select(time);
    }
}
