package com.ruoyi.crops.service.impl;

import cn.hutool.http.HttpUtil;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.ruoyi.crops.domain.MachineParameter;
import com.ruoyi.crops.domain.UserInfo;
import com.ruoyi.crops.domain.gsonBean.GsonMachineParameterBean;
import com.ruoyi.crops.domain.vo.MachineParameterVo;
import com.ruoyi.crops.service.IMachineParameterService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class MachineParameterServiceImpl implements IMachineParameterService {
    @Autowired
    private RedisTemplate redisTemplate;

    @Value("${url.machine}")
    private String url;

    /**
     * 获取水肥机全部状态参数
     * @return
     * @throws Exception
     */
    @Override
    public MachineParameter getMachineParameter() throws Exception {

        Map<String, Object> map = new HashMap<>();//存放参数
        UserInfo userInfo = (UserInfo) redisTemplate.opsForValue().get("cropsUserInfo");
//        map.put("id", userInfo.getId());
        map.put("id", "7f3f70c7cd134881b84d1db36b20c416");
        map.put("deviceId", "00093495000000000002");
        HashMap<String, String> headers = new HashMap<>();//存放请求头，可以存放多个请求头
//        headers.put("token", userInfo.getToken());
        headers.put("token", "BAC5284E7938B521E888E9B3CB6B5DA9");
        //发送get请求并接收响应数据
        String result = HttpUtil.createGet(url).addHeaders(headers).form(map).execute().body();

        Gson gson = new GsonBuilder().setDateFormat("yyyy/MM/dd HH:mm:ss").create();
        //格式化json，封装到GsonDeviceSenseBean类型
        GsonMachineParameterBean model = gson.fromJson(result, GsonMachineParameterBean.class);

        MachineParameter machineParameter = new MachineParameter();
        //反射获取data里所有对象
        Field[] field = model.data.getClass().getDeclaredFields();

        for (int i = 0; i < field.length - 1; i++) {
            //遍历对象的属性名
            String name = field[i].getName();
            //获取相应对象的get方法
            Method m = model.data.getClass().getMethod("get" + name);
            //执行get方法，得到property对象
            GsonMachineParameterBean.property value = (GsonMachineParameterBean.property) m.invoke(model.data);
            //获取property对象里的value
            String propertyValve = value.propertyValve;

            if (propertyValve != null) {
                //获取要set对象的对应类型
                Field f = machineParameter.getClass().getDeclaredField(name);
                String type = f.getType().getName();
                Class[] classes = new Class[0];
                Object[] objects = new Object[0];
                if (type.equals("java.lang.Double")) {
                    classes = new Class[]{Double.class};
                    objects = new Object[]{new Double(Double.parseDouble(propertyValve))};
                } else if (type.equals("java.lang.String")) {
                    classes = new Class[]{String.class};
                    objects = new Object[]{new String(propertyValve)};
                }
                Method mset = machineParameter.getClass().getMethod("set" + name, classes);
                mset.invoke(machineParameter, objects);
            }
        }
        return machineParameter;
    }

    /**
     * 获取ph值，流量，压力值
     * @return
     * @throws Exception
     */
    @Override
    public MachineParameterVo selectParameter() throws Exception {
        MachineParameter machineParameter = getMachineParameter();
        MachineParameterVo vo = new MachineParameterVo();
        BeanUtils.copyProperties(machineParameter,vo);
        return vo;

    }
}
