package com.ruoyi.crops.service.impl;

import com.ruoyi.common.utils.MyPoint;
import com.ruoyi.crops.service.IMapDrawPolygonService;

import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class MapDrawPolygonServiceImpl implements IMapDrawPolygonService {

//    public String DrawPolygon(List<MyPoint> points) {
//
//        String wkt = point2WktPolygon(convexHull(points));
////        AjaxResult a = new AjaxResult();
////        a.put("WKT",wkt);
////        a.put("area",getAreaByWkt(wkt));
////        return AjaxResult.success(a);
//        return wkt;
//    }

    @Override
    public List<MyPoint> DrawPolygon(List<MyPoint> points) {
        return convexHull(points);
    }

    public static List<MyPoint> convexHull(List<MyPoint> points) {
        //小于三个点的情况，直接返回
        if (points.size() < 3) {
            return points;
        }

        MyPoint first = points.get(0);
        // 找到最左下角的点
        for (int i = 0; i < points.size(); i++) {
            if (points.get(i).getY() < first.getY() || (points.get(i).getY() == first.getY() && points.get(i).getX() < first.getX())) {
                first = points.get(i);
            }
        }

        List<MyPoint> sortedList = angleSorting(first, points);
//        logger.info("凸包计算-最下面偏左的点位是：{}", first);
//        logger.info("凸包计算-角排序结果：{}", JSONObject.toJSONString(sortedList));
        int firstIndex = 0;
        for (int i = 0; i < sortedList.size(); i++) {
            if (first.getX() == sortedList.get(i).getX() && first.getY() == sortedList.get(i).getY()) {
                firstIndex = i;
            }
        }
        // 结果集合
        List<MyPoint> convexHullPoints = new ArrayList<>();
        convexHullPoints.add(first);
        int nextIndex = firstIndex + 1 == sortedList.size() ? 0 : firstIndex + 1;
        MyPoint next = new MyPoint();
        while (next != first) {
            next = sortedList.get(nextIndex);

            while (true) {
                if (convexHullPoints.size() < 2) {
                    convexHullPoints.add(next);
                    break;
                }
                if (isEnabled(convexHullPoints.get(convexHullPoints.size() - 2), convexHullPoints.get(convexHullPoints.size() - 1), next)) {
                    convexHullPoints.add(next);
                    break;
                } else {
                    convexHullPoints.remove(convexHullPoints.size() - 1);
                }
            }
            nextIndex = nextIndex + 1 == sortedList.size() ? 0 : nextIndex + 1;
        }
        convexHullPoints = convexHullPoints.stream().distinct().collect(Collectors.toList());
//        logger.info("凸包计算-结果:{}", JSONObject.toJSONString(convexHullPoints));
//        logger.info("凸包结算-结果绘制成线:{}", point2WktLine(convexHullPoints));
        return convexHullPoints;
    }

    public static boolean isEnabled(MyPoint A, MyPoint B, MyPoint C) {
        double mulCross = A.getX() * B.getY() - A.getY() * B.getX() + B.getX() * C.getY() - B.getY() * C.getX() + C.getX() * A.getY() - C.getY() * A.getX();
        if (mulCross >= 0) {
            return true;
        } else {
            return false;
        }
    }
    public static class MyHashMap<k,v> extends HashMap{
        @Override
        public Object put(Object key, Object value) {
            if (containsKey(key)){
                return null;
            }
            return super.put(key, value);
        }
    }
    /**
     * 以原点为中心进行角排序
     *
     * @param points 点位
     * @return 排序的点位
     */
    public static List<MyPoint> angleSorting(MyPoint first, List<MyPoint> points) {
        points = points.stream().sorted(Comparator.comparing(MyPoint::getX)).collect(Collectors.toList());
        points = points.stream().sorted(Comparator.comparing(MyPoint::getY)).collect(Collectors.toList());
        MyHashMap<Double, MyPoint> pointMap = new MyHashMap<>();
        List<Double> angles = new ArrayList<>();
        for (MyPoint point : points) {
            double angle = Math.atan2(point.getY() - first.getY(), point.getX() - first.getX()) * 180.0 / Math.PI;
            if (angle < 0) {
                angle += 360.0;
            }
            pointMap.put(angle, point);
            angles.add(angle);
        }
        angles = angles.stream().sorted().collect(Collectors.toList());
        List<MyPoint> result = new ArrayList<>();
        for (Double angle : angles) {
            result.add((MyPoint) pointMap.get(angle));
        }

//        result.add(first);
        return result;
    }

    public static String point2WktPolygon(List<MyPoint> points) {
        StringBuilder sb = new StringBuilder("POLYGON((");
        for (MyPoint point : points) {
            sb.append(point.getX()).append(" ").append(point.getY()).append(",");
        }
        sb.append(points.get(0).getX()).append(" ").append(points.get(0).getY()).append("))");

        return sb.toString();
    }

    public static String point2WktLine(List<MyPoint> points) {
        StringBuilder sb = new StringBuilder("LINESTRING(");
        for (MyPoint point : points) {
            sb.append(point.getX()).append(" ").append(point.getY()).append(", ");
        }
        sb.append(points.get(0).getX()).append(" ").append(points.get(0).getY()).append(")");

        return sb.toString();
    }

    public static String point2WktPoint(List<MyPoint> points) {
        StringBuilder sb = new StringBuilder("MULTIPOINT(");
        for (MyPoint point : points) {
            sb.append(point.getX()).append(" ").append(point.getY()).append(", ");
        }
        sb.append(points.get(0).getX()).append(" ").append(points.get(0).getY()).append(")");
        return sb.toString();
    }
}
